/*
 * Copyright 2009 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.lisp.test.sos;

import net.morilib.lisp.Scheme;
import net.morilib.lisp.Undef;
import net.morilib.lisp.test.TCSubr;

public class ClassTest extends TCSubr {
	
	public void testClass1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance)" +
				"   (ins2)" +
				"   (cls1 :allocation :class)" +
				"   (ecl1 :allocation :each-subclass)))");
		l.exec ("(define i1 (make <t1>))");
		eq   (l,"(slot-ref <t1> 'cls1)", Undef.UNDEF);
		eq   (l,"(slot-ref <t1> 'ecl1)", Undef.UNDEF);
		eq   (l,"(slot-ref i1   'cls1)", Undef.UNDEF);
		eq   (l,"(slot-ref i1   'ecl1)", Undef.UNDEF);
		eq   (l,"(slot-ref i1   'ins1)", Undef.UNDEF);
		eq   (l,"(slot-ref i1   'ins2)", Undef.UNDEF);
		lperr(l,"(slot-ref <t1> 'ins1)");
		lperr(l,"(slot-ref <t1> 'ins2)");
		
		lperr(l,"(define-class <t2> () " +
				"  ((ins1 :invalid)))");
		lperr(l,"(define-class <t3> () " +
				"  ((ins1 :allocation :invalid)))");
	}
	
	public void testClass2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance :init-value 100)" +
				"   (ins2 :init-value 200)" +
				"   (cls1 :allocation :class :init-value 300)" +
				"   (ecl1 :allocation :each-subclass :init-value 400)))");
		l.exec ("(define i1 (make <t1>))");
		eqi  (l,"(slot-ref <t1> 'cls1)", 300);
		eqi  (l,"(slot-ref <t1> 'ecl1)", 400);
		eqi  (l,"(slot-ref i1   'cls1)", 300);
		eqi  (l,"(slot-ref i1   'ecl1)", 400);
		eqi  (l,"(slot-ref i1   'ins1)", 100);
		eqi  (l,"(slot-ref i1   'ins2)", 200);
		lperr(l,"(slot-ref <t1> 'ins1)");
		lperr(l,"(slot-ref <t1> 'ins2)");
	}
	
	public void testSlotRef1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance :init-value 100)" +
				"   (ins2 :init-value 200)" +
				"   (cls1 :allocation :class :init-value 300)" +
				"   (ecl1 :allocation :each-subclass :init-value 400)))");
		l.exec ("(define i1 (make <t1>))");
		eqi  (l,"(slot-ref <t1> 'cls1)", 300);
		eqi  (l,"(slot-ref <t1> 'ecl1)", 400);
		eqi  (l,"(slot-ref i1   'cls1)", 300);
		eqi  (l,"(slot-ref i1   'ecl1)", 400);
		eqi  (l,"(slot-ref i1   'ins1)", 100);
		eqi  (l,"(slot-ref i1   'ins2)", 200);
	}
	
	public void testSlotSetS1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance)" +
				"   (cls1 :allocation :class)" +
				"   (ecl1 :allocation :each-subclass)))");
		l.exec ("(define i1 (make <t1>))");
		l.exec ("(slot-set! <t1> 'cls1 111)");
		l.exec ("(slot-set! <t1> 'ecl1 222)");
		l.exec ("(slot-set! i1   'ins1 333)");
		eqi  (l,"(slot-ref <t1> 'cls1)", 111);
		eqi  (l,"(slot-ref <t1> 'ecl1)", 222);
		eqi  (l,"(slot-ref i1   'ins1)", 333);
	}
	
	public void testIsClass1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () ())");
		eq   (l,"(class? <t1>)", T);
		eq   (l,"(class? <top>)", T);
		eq   (l,"(class? <number>)", T);
		eq   (l,"(class? #t)", F);
		eq   (l,"(class? 'a)", F);
	}
	
	public void testClassGetter() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance :init-value 100" +
				"         :getter gti)" +
				"   (cls1 :allocation :class :init-value 300" +
				"         :getter gtc)" +
				"   (ecl1 :allocation :each-subclass :init-value 400" +
				"         :getter gte)))");
		l.exec ("(define i1 (make <t1>))");
		eqi  (l,"(gti i1)", 100);
		eqi  (l,"(gtc i1)", 300);
		eqi  (l,"(gte i1)", 400);
	}
	
	public void testClassSetter() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance :init-value 100" +
				"         :setter sti)" +
				"   (cls1 :allocation :class :init-value 300" +
				"         :setter stc)" +
				"   (ecl1 :allocation :each-subclass :init-value 400" +
				"         :setter ste)))");
		l.exec ("(define i1 (make <t1>))");
		l.exec ("(sti i1 111)");
		l.exec ("(stc i1 222)");
		l.exec ("(ste i1 333)");
		eqi  (l,"(slot-ref i1 'ins1)", 111);
		eqi  (l,"(slot-ref i1 'cls1)", 222);
		eqi  (l,"(slot-ref i1 'ecl1)", 333);
	}
	
	public void testClassAccessor() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((ins1 :allocation :instance :init-value 100" +
				"         :accessor aci)" +
				"   (cls1 :allocation :class :init-value 300" +
				"         :accessor acc)" +
				"   (ecl1 :allocation :each-subclass :init-value 400" +
				"         :accessor ace)))");
		l.exec ("(define i1 (make <t1>))");
		l.exec ("(set! (aci i1) 111)");
		l.exec ("(set! (acc i1) 222)");
		l.exec ("(set! (ace i1) 333)");
		eqi  (l,"(aci i1)", 111);
		eqi  (l,"(acc i1)", 222);
		eqi  (l,"(ace i1)", 333);
	}
	
	public void testClassInherit1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <pane> () ())");
		l.exec ("(define-class <scrolling-mixin> () ())");
		l.exec ("(define-class <scrollable-pane>" +
				"  (<pane> <scrolling-mixin>) ())");
		l.exec ("(define-class <editing-mixin> () ())");
		l.exec ("(define-class <editable-pane>" +
				"  (<pane> <editing-mixin>) ())");
		l.exec ("(define-class <editable-scrollable-pane>" +
				"  (<scrollable-pane> <editable-pane>) ())");
		equal(l,"(map class-name" +
				"  (class-precedence-list <editable-scrollable-pane>))",
				list(sym("<editable-scrollable-pane>"), 
						sym("<scrollable-pane>"),
						sym("<editable-pane>"),
						sym("<pane>"),
						sym("<scrolling-mixin>"),
						sym("<editing-mixin>"),
						sym("<object>"),
						sym("<top>")));
	}
	
	public void testClassInherit2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <fruit> () ())");
		l.exec ("(define-class <apple> (<fruit>) ())");
		lperr(l,"(define-class <new-class> (<fruit> <apple>) ())");
	}
	
	public void testClassInherit3() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> () " +
				"  ((a :allocation :instance      :init-value 101)" +
				"   (b :allocation :instance      :init-value 102)" +
				"   (c :allocation :class         :init-value 103)" +
				"   (d :allocation :class         :init-value 104)" +
				"   (e :allocation :each-subclass :init-value 105)" +
				"   (f :allocation :each-subclass :init-value 106)))");
		l.exec ("(define-class <t2> () " +
				"  ((g :allocation :instance      :init-value 201)" +
				"   (h :allocation :instance      :init-value 202)" +
				"   (i :allocation :class         :init-value 203)" +
				"   (j :allocation :class         :init-value 204)" +
				"   (k :allocation :each-subclass :init-value 205)" +
				"   (l :allocation :each-subclass :init-value 206)))");
		l.exec ("(define-class <t3> (<t1> <t2>) " +
				"  ((a :allocation :instance      :init-value 301)" +
				"   (g :allocation :instance      :init-value 302)" +
				"   (c :allocation :instance      :init-value 303)" +
				"   (e :allocation :instance      :init-value 304)" +
				"   (m :allocation :instance      :init-value 305)" +
				"   (j :allocation :class         :init-value 306)" +
				"   (f :allocation :class         :init-value 307)" +
				"   (n :allocation :class         :init-value 308)" +
				"   (d :allocation :each-subclass :init-value 309)" +
				"   (o :allocation :each-subclass :init-value 311)))");
		lperr(l,"(slot-ref <t3> 'c)");
		eqi  (l,"(slot-ref <t3> 'd)", 309);
		lperr(l,"(slot-ref <t3> 'e)");
		eqi  (l,"(slot-ref <t3> 'f)", 307);
		eqi  (l,"(slot-ref <t3> 'i)", 203);
		eqi  (l,"(slot-ref <t3> 'j)", 306);
		eq   (l,"(slot-ref <t3> 'k)", Undef.UNDEF);
		eq   (l,"(slot-ref <t3> 'l)", Undef.UNDEF);
		eqi  (l,"(slot-ref <t3> 'n)", 308);
		eqi  (l,"(slot-ref <t3> 'o)", 311);
		l.exec ("(define i1 (make <t3>))");
		eqi  (l,"(slot-ref i1 'a)", 301);
		eqi  (l,"(slot-ref i1 'b)", 102);
		eqi  (l,"(slot-ref i1 'c)", 303);
		eqi  (l,"(slot-ref i1 'd)", 309);
		eqi  (l,"(slot-ref i1 'e)", 304);
		eqi  (l,"(slot-ref i1 'f)", 307);
		eqi  (l,"(slot-ref i1 'g)", 302);
		eqi  (l,"(slot-ref i1 'h)", 202);
		eqi  (l,"(slot-ref i1 'i)", 203);
		eqi  (l,"(slot-ref i1 'j)", 306);
		eq   (l,"(slot-ref i1 'k)", Undef.UNDEF);
		eq   (l,"(slot-ref i1 'l)", Undef.UNDEF);
		eqi  (l,"(slot-ref i1 'm)", 305);
		eqi  (l,"(slot-ref i1 'n)", 308);
		eqi  (l,"(slot-ref i1 'o)", 311);
	}
	
	public void testMake1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> ()" +
				"  ((a :init-value 400 :init-keyword :i1 :getter gt1)" +
				"   (b :init-value 500 :init-keyword :i2 :getter gt2)))");
		l.exec ("(define i1 (make <t1> :i2 200))");
		eqi  (l,"(gt1 i1)", 400);
		eqi  (l,"(gt2 i1)", 200);
	}
	
	public void testMake2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> ()" +
				"  ((a :init-value 400 :init-keyword :i1 :getter gt1)" +
				"   (b :init-value 500 :init-keyword :i2 :getter gt2)))");
		l.exec ("(define-class <t2> (<t1>)" +
				"  ((c :init-value 400 :init-keyword :i3)" +
				"   (d :init-value 500 :init-keyword :i4)))");
		l.exec ("(define i1 (make <t2> :i2 200 :i4 300))");
		eqi  (l,"(slot-ref i1 'b)", 200);
		eqi  (l,"(slot-ref i1 'd)", 300);
	}
	
	public void testInitialize1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> ()" +
				"  ((a :init-value 400 :init-keyword :i1 :getter gt1)" +
				"   (b :init-value 500 :init-keyword :i2 :getter gt2)))");
		l.exec ("(define-method initialize ((self <t1>) args)" +
				"  (slot-set! self 'a 123)" +
				"  (slot-set! self 'b 456))");
		l.exec ("(define i1 (make <t1> :i2 200))");
		eqi  (l,"(gt1 i1)", 123);
		eqi  (l,"(gt2 i1)", 456);
	}
	
	public void testInitialize2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <t1> ()" +
				"  ((a :init-value 400 :init-keyword :i1 :getter gt1)" +
				"   (b :init-value 500 :init-keyword :i2 :getter gt2)))");
		l.exec ("(define-class <t2> (<t1>)" +
				"  ((c :init-value 400 :init-keyword :i3 :getter gt3)" +
				"   (d :init-value 500 :init-keyword :i4 :getter gt4)))");
		l.exec ("(define-method initialize ((self <t1>) args)" +
				"  (next-method)" +
				"  (slot-set! self 'a 123)" +
				"  (slot-set! self 'b 456))");
		l.exec ("(define-method initialize ((self <t2>) args)" +
				"  (next-method)" +
				"  (slot-set! self 'c 234)" +
				"  (slot-set! self 'd 567))");
		l.exec ("(define i1 (make <t2> :i2 200))");
		eqi  (l,"(gt1 i1)", 123);
		eqi  (l,"(gt2 i1)", 456);
		eqi  (l,"(gt3 i1)", 234);
		eqi  (l,"(gt4 i1)", 567);
	}
	
	public void testMetaClass1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :init-value 100)))");
		l.exec ("(define-class <t1> ()" +
				"  () :metaclass <m1>)");
		l.exec ("(define i1 (make <t1>))");
		eqi  (l,"(slot-ref <t1> 'number)", 100);
		lperr(l,"(slot-ref i1 'number)");
	}
	
	public void testMetaClass2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :init-value 100)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> (<t1>) ())");
		l.exec ("(define i1 (make <t2>))");
		eqi  (l,"(slot-ref <t2> 'number)", 100);
		lperr(l,"(slot-ref i1 'number)");
	}
	
	public void testMetaClass3() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :init-value 100)))");
		l.exec ("(define-class <m2> (<class>)" +
				"  ((num2   :init-value 200)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> () () :metaclass <m2>)");
		l.exec ("(define-class <t3> (<t1> <t2>) ())");
		l.exec ("(define i1 (make <t3>))");
		eqi  (l,"(slot-ref <t3> 'number)", 100);
		eqi  (l,"(slot-ref <t3> 'num2)",   200);
		lperr(l,"(slot-ref i1 'number)");
		lperr(l,"(slot-ref i1 'num2)");
	}
	
	public void testMetaClass4() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :allocation :class :init-value 100)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> (<t1>) ())");
		l.exec ("(define i1 (make <t2>))");
		eqi  (l,"(slot-ref <t2> 'number)", 100);
		lperr(l,"(slot-ref i1 'number)");
	}
	
	public void testMetaClass5() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :allocation :class :init-value 100)))");
		l.exec ("(define-class <m2> (<class>)" +
				"  ((num2   :allocation :class :init-value 200)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> () () :metaclass <m2>)");
		l.exec ("(define-class <t3> (<t1> <t2>) ())");
		l.exec ("(define i1 (make <t3>))");
		eqi  (l,"(slot-ref <t3> 'number)", 100);
		eqi  (l,"(slot-ref <t3> 'num2)",   200);
		lperr(l,"(slot-ref i1 'number)");
		lperr(l,"(slot-ref i1 'num2)");
	}
	
	public void testMetaClass6() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :allocation :class :init-value 100)))");
		l.exec ("(define-class <m2> (<class>)" +
				"  ((num2   :allocation :class :init-value 200)))");
		l.exec ("(define-class <m3> (<m2>)" +
				"  ((num3   :allocation :class :init-value 300)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> () () :metaclass <m3>)");
		l.exec ("(define-class <t3> (<t1> <t2>) ())");
		l.exec ("(define i1 (make <t3>))");
		eqi  (l,"(slot-ref <t3> 'number)", 100);
		eqi  (l,"(slot-ref <t3> 'num2)",   200);
		eqi  (l,"(slot-ref <t3> 'num3)",   300);
		lperr(l,"(slot-ref i1 'number)");
		lperr(l,"(slot-ref i1 'num2)");
		lperr(l,"(slot-ref i1 'num3)");
	}
	
	public void testMetaClass7() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :allocation :each-subclass" +
				"           :init-value 100)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> (<t1>) ())");
		l.exec ("(define i1 (make <t2>))");
		eqi  (l,"(slot-ref <t2> 'number)", 100);
		lperr(l,"(slot-ref i1 'number)");
	}
	
	public void testMetaClass8() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :allocation :each-subclass" +
				"           :init-value 100)))");
		l.exec ("(define-class <m2> (<class>)" +
				"  ((num2   :allocation :each-subclass" +
				"           :init-value 200)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> () () :metaclass <m2>)");
		l.exec ("(define-class <t3> (<t1> <t2>) ())");
		l.exec ("(define i1 (make <t3>))");
		eq   (l,"(slot-ref <t3> 'number)", Undef.UNDEF);
		eq   (l,"(slot-ref <t3> 'num2)",   Undef.UNDEF);
		lperr(l,"(slot-ref i1 'number)");
		lperr(l,"(slot-ref i1 'num2)");
	}
	
	public void testMetaClass9() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((number :allocation :each-subclass" +
				"           :init-value 100)))");
		l.exec ("(define-class <m2> (<class>)" +
				"  ((num2   :allocation :each-subclass" +
				"           :init-value 200)))");
		l.exec ("(define-class <m3> (<m2>)" +
				"  ((num3   :allocation :each-subclass" +
				"           :init-value 300)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-class <t2> () () :metaclass <m3>)");
		l.exec ("(define-class <t3> (<t1> <t2>) ())");
		l.exec ("(define i1 (make <t3>))");
		eq   (l,"(slot-ref <t3> 'number)", Undef.UNDEF);
		eq   (l,"(slot-ref <t3> 'num2)",   Undef.UNDEF);
		eq   (l,"(slot-ref <t3> 'num3)",   Undef.UNDEF);
		lperr(l,"(slot-ref i1 'number)");
		lperr(l,"(slot-ref i1 'num2)");
		lperr(l,"(slot-ref i1 'num3)");
	}
	
	public void testMetaClass10() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-class <m1> (<class>)" +
				"  ((num :init-value 0)))");
		l.exec ("(define-class <t1> () () :metaclass <m1>)");
		l.exec ("(define-method make ((kls <m1>) . initargs)" +
				"  (slot-set! <t1> 'num (+ (slot-ref <t1> 'num) 1))" +
				"  (next-method))");
		eqi  (l,"(slot-ref <t1> 'num)", 0);
		l.exec ("(define i1 (make <t1>))");
		eqi  (l,"(slot-ref <t1> 'num)", 1);
		l.exec ("(define i1 (make <t1>))");
		eqi  (l,"(slot-ref <t1> 'num)", 2);
	}
	
}
