/*
 * Copyright 2009 Yuichiro Moriguchi
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.morilib.lisp.test.sos;

import net.morilib.lisp.Scheme;
import net.morilib.lisp.test.TCSubr;

public class GenericTest extends TCSubr {
	
	public void testGeneric1() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <integer>))" +
				"  (+ a 1))");
		eqi  (l,"(mt1 1)", 2);
		lperr(l,"(mt1 1/2)");
	}
	
	public void testGeneric2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <integer>))  (+ a 1))");
		l.exec ("(define-method mt1 ((a <rational>)) (* a 2))");
		l.exec ("(define-method mt1 ((a <string>))" +
				"  (string->number a))");
		eqi  (l,"(mt1 1)", 2);
		eqi  (l,"(mt1 1/2)", 1);
		eqi  (l,"(mt1 \"111\")", 111);
		lperr(l,"(mt1 #\\a)");
	}
	
	public void testGeneric3() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <integer>))" +
				"  (+ (next-method) 1))");
		l.exec ("(define-method mt1 ((a <rational>)) (* a 2))");
		eqi  (l,"(mt1 1)", 3);
		eqi  (l,"(mt1 1/2)", 1);
	}
	
	public void testGeneric4() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <integer>))" +
				"  (+ (next-method (+ a 1)) 1))");
		l.exec ("(define-method mt1 ((a <rational>)) (* a 2))");
		eqi  (l,"(mt1 1)", 5);
		eqi  (l,"(mt1 1/2)", 1);
	}
	
	public void testGeneric5() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <top>) (b <top>) (c <top>))" +
				"  (list a b c))");
		l.exec ("(define-method mt1 ((a <integer>)  (b <integer>))" +
				"  (+ a b))");
		l.exec ("(define-method mt1 ((a <rational>) (b <integer>))" +
				"  (* a b))");
		l.exec ("(define-method mt1 ((a <integer>)  (b <rational>))" +
				"  (/ a b))");
		l.exec ("(define-method mt1 ((a <rational>) (b <rational>))" +
				"  (- a b))");
		eqi  (l,"(mt1 1 1)", 2);
		eqi  (l,"(mt1 3/2 2)", 3);
		eqi  (l,"(mt1 2 2/3)", 3);
		eqi  (l,"(mt1 3/2 1/2)", 1);
		equal(l,"(mt1 1 2 3)", list(1, 2, 3));
	}
	
	public void testGeneric6() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <number>) (b <number>))" +
				"  (+ a b))");
		l.exec ("(define-method mt1 ((a <number>) b)" +
				"  (list a b))");
		equal(l,"(mt1 1+i 2+i)", newC(3, 2));
		equal(l,"(mt1 1 'b)", list(1, sym("b")));
	}
	
	public void testGeneric7() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <number>) (b <number>))" +
				"  (+ a b))");
		l.exec ("(define-method mt1 ((a <number>) . b)" +
				"  (cons a b))");
		equal(l,"(mt1 1+i 2+i)", newC(3, 2));
		equal(l,"(mt1 1 2 3)", list(1, 2, 3));
	}
	
	public void testGeneric7_2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <number>) . b)" +
				"  (cons a b))");
		l.exec ("(define-method mt1 ((a <number>) (b <number>))" +
				"  (+ a b))");
		equal(l,"(mt1 1+i 2+i)", newC(3, 2));
		equal(l,"(mt1 1 2 3)", list(1, 2, 3));
	}
	
	public void testGeneric8() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <number>) (b <number>))" +
				"  (+ a b))");
		l.exec ("(define-method mt1 a" +
				"  (if (list? a) (reverse a) (list a)))");
		equal(l,"(mt1 1+i 2+i)", newC(3, 2));
		equal(l,"(mt1 1 2 3)", list(3, 2, 1));
		equal(l,"(mt1 1)", list(1));
	}
	
	public void testGeneric8_2() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 a" +
				"  (if (list? a) (reverse a) (list a)))");
		l.exec ("(define-method mt1 ((a <number>) (b <number>))" +
				"  (+ a b))");
		equal(l,"(mt1 1+i 2+i)", newC(3, 2));
		equal(l,"(mt1 1 2 3)", list(3, 2, 1));
		equal(l,"(mt1 1)", list(1));
	}
	
	public void testGeneric9() {
		Scheme l = Scheme.newInstance();
		
		l.exec ("(define-method mt1 ((a <number>) (b <number>))" +
				"  (+ a b))");
		l.exec ("(define-method mt1 ((a <number>) b)" +
				"  (list a b))");
		l.exec ("(remove-method mt1 (<number> <number>))");
		equal(l,"(mt1 1 2)", list(1, 2));
		equal(l,"(mt1 1 'b)", list(1, sym("b")));
	}
	
}
